@@ -4,17 +4,19 @@ import icepool
 import icepool.math
 import icepool.creation_args
 from icepool.counts import Counts, CountsKeysView, CountsValuesView, CountsItemsView
+from icepool.population import Population
 
+from collections import defaultdict
 from functools import cached_property
+import operator
 
-from typing import Any, Iterator
-from collections.abc import Mapping, Sequence
+from typing import Any, Callable, Iterator, Mapping, MutableMapping, Sequence
 
 
-class Deck(Mapping[Any, int]):
-    """EXPERIMENTAL: Represents a deck to be sampled without replacement.
+class Deck(Population):
+    """EXPERIMENTAL: Sampling without replacement (within a single evaluation).
 
-    API and naming WIP.
+    Quantities represent duplicates.
     """
 
     _data: Counts
@@ -23,17 +25,17 @@ class Deck(Mapping[Any, int]):
     def __new__(cls,
                 outcomes: Mapping[Any, int] | Sequence,
                 times: Sequence[int] | int = 1) -> 'Deck':
-        """Constructor for a deck.
+        """Constructor for a `Deck`.
 
         Args:
-            outcomes: The cards of the deck. This can be one of the following:
-                * A `Mapping` from outcomes to dups.
+            outcomes: The cards of the `Deck`. This can be one of the following:
+                * A `Mapping` from outcomes to quantities.
                 * A sequence of outcomes.
 
                 Note that `Die` and `Deck` both count as `Mapping`s.
 
                 Each card may be one of the following:
-                * A `Mapping` from outcomes to dups.
+                * A `Mapping` from outcomes to quantities.
                     The outcomes of the `Mapping` will be "flattened" into the
                     result. This option will be taken in preference to treating
                     the `Mapping` itself as an outcome even if the `Mapping`
@@ -49,9 +51,9 @@ class Deck(Mapping[Any, int]):
                     Each outcome must be hashable, and the
                     set of outcomes must be totally orderable (after expansion).
                     The same outcome can appear multiple times, in which case
-                    the corresponding dups will be accumulated.
+                    the corresponding quantities will be accumulated.
             times: Multiplies the number of times each element of `outcomes`
-                will be put into the deck.
+                will be put into the `Deck`.
                 `times` can either be a sequence of the same length as
                 `outcomes` or a single `int` to apply to all elements of
                 `outcomes`.
@@ -71,9 +73,20 @@ class Deck(Mapping[Any, int]):
         data = icepool.creation_args.expand_args_for_deck(outcomes, times)
         return Deck._new_deck(data)
 
+    def unary_op_non_elementwise(self, op: Callable, *args, **kwargs) -> 'Deck':
+        """As `unary_op()`, but not elementwise.
+
+        This is used for `marginals()`.
+        """
+        data: MutableMapping[Any, int] = defaultdict(int)
+        for outcome, quantity in self.items():
+            new_outcome = op(outcome, *args, **kwargs)
+            data[new_outcome] += quantity
+        return Deck(data)
+
     @classmethod
     def _new_deck(cls, data: Counts) -> 'Deck':
-        """Creates a new deck using already-processed arguments.
+        """Creates a new `Deck` using already-processed arguments.
 
         Args:
             data: At this point, this is a Counts.
@@ -82,34 +95,12 @@ class Deck(Mapping[Any, int]):
         self._data = data
         return self
 
-    def outcomes(self) -> CountsKeysView:
-        """The outcomes of the deck in sorted order.
-
-        These are also the `keys` of the deck as a `Mapping`.
-        Prefer to use the name `outcomes`.
-        """
+    def keys(self) -> CountsKeysView:
         return self._data.keys()
 
-    keys = outcomes
-
-    def min_outcome(self):
-        """Returns the minimum possible outcome of this deck."""
-        return self.outcomes()[0]
-
-    def max_outcome(self):
-        """Returns the maximum possible outcome of this deck."""
-        return self.outcomes()[-1]
-
-    def dups(self) -> CountsValuesView:
-        """The dups of the deck in outcome order.
-
-        These are also the `values` of the deck as a `Mapping`.
-        Prefer to use the name `dups`.
-        """
+    def values(self) -> CountsValuesView:
         return self._data.values()
 
-    values = dups
-
     def items(self) -> CountsItemsView:
         return self._data.items()
 
@@ -122,28 +113,22 @@ class Deck(Mapping[Any, int]):
     def __len__(self) -> int:
         return len(self._data)
 
-    @cached_property
-    def _num_cards(self) -> int:
-        return sum(self._data.values())
-
-    def num_cards(self) -> int:
-        """The total number of cards in this deck (including dups)."""
-        return self._num_cards
+    size = icepool.Population.denominator
 
     @cached_property
     def _popped_min(self) -> tuple['Deck', int]:
-        return self._new_deck(self._data.remove_min()), self.dups()[0]
+        return self._new_deck(self._data.remove_min()), self.quantities()[0]
 
     def _pop_min(self) -> tuple['Deck', int]:
-        """Returns a deck with the min outcome removed."""
+        """A `Deck` with the min outcome removed."""
         return self._popped_min
 
     @cached_property
     def _popped_max(self) -> tuple['Deck', int]:
-        return self._new_deck(self._data.remove_max()), self.dups()[-1]
+        return self._new_deck(self._data.remove_max()), self.quantities()[-1]
 
     def _pop_max(self) -> tuple['Deck', int]:
-        """Returns a deck with the max outcome removed."""
+        """A `Deck` with the max outcome removed."""
         return self._popped_max
 
     def deal(self, *hand_sizes: int) -> 'icepool.Deal':
@@ -153,6 +138,28 @@ class Deck(Mapping[Any, int]):
         """
         return icepool.Deal(self, *hand_sizes)
 
+    class _Marginals():
+        """Helper class for implementing `marginals()`."""
+
+        _deck: 'Deck'
+
+        def __init__(self, deck: 'Deck', /):
+            self._deck = deck
+
+        def __getitem__(self, dims, /) -> 'Deck':
+
+            return self._deck.unary_op_non_elementwise(operator.getitem, dims)
+
+    @property
+    def marginals(self):
+        """A property that applies the `[]` operator to outcomes.
+
+        This is not performed elementwise on tuples, so that this can be used
+        to slice tuples. For example, `deck.marginals[:2]` will marginalize the
+        first two elements of tuples.
+        """
+        return Deck._Marginals(self)
+
     @cached_property
     def _key_tuple(self) -> tuple:
         return Deck, tuple(self.items())
@@ -168,3 +175,8 @@ class Deck(Mapping[Any, int]):
 
     def __hash__(self) -> int:
         return self._hash
+
+    def __repr__(self) -> str:
+        inner = ', '.join(
+            f'{outcome}: {quantity}' for outcome, quantity in self.items())
+        return type(self).__qualname__ + '({' + inner + '})'
