@@ -7,27 +7,26 @@ from functools import cache
 import itertools
 import math
 
-from typing import Any, Callable
-from collections.abc import Sequence
+from typing import Any, Callable, Sequence
 
 
 @cache
-def standard(num_sides: int, /) -> 'icepool.Die':
+def standard(sides: int, /) -> 'icepool.Die':
     """A standard die.
 
-    Specifically, the outcomes are `int`s from `1` to `num_sides` inclusive,
-    with weight 1 each.
+    Specifically, the outcomes are `int`s from `1` to `sides` inclusive,
+    with quantity 1 each.
 
     Don't confuse this with `icepool.Die()`:
 
-    * `icepool.Die([6])`: A die that always rolls the integer 6.
+    * `icepool.Die([6])`: A `Die` that always rolls the integer 6.
     * `icepool.d(6)`: A d6.
     """
-    if not isinstance(num_sides, int):
+    if not isinstance(sides, int):
         raise TypeError('Argument to standard() must be an int.')
-    elif num_sides < 1:
+    elif sides < 1:
         raise ValueError('Standard die must have at least one side.')
-    return icepool.Die(range(1, num_sides + 1))
+    return icepool.Die(range(1, sides + 1))
 
 
 d = standard
@@ -49,73 +48,82 @@ def __getattr__(key: str):
     raise AttributeError(key)
 
 
-def bernoulli(n: int, d: int, /) -> 'icepool.Die':
-    """A die that rolls `True` with chance `n / d`, and `False` otherwise. """
-    return icepool.Die({False: d - n, True: n})
+def bernoulli(numerator: int, denominator: int, /) -> 'icepool.Die':
+    """A `Die` that rolls `True` with probability `n / d`, and `False` otherwise.
+
+    The `Die` will have both `True` and `False` as outcomes even if one has
+    zero quantity.
+    """
+    return icepool.Die({False: denominator - numerator, True: numerator})
 
 
 coin = bernoulli
 
 
-def from_cweights(outcomes: Sequence, cweights: Sequence[int]) -> 'icepool.Die':
-    """Constructs a die from cumulative weights. """
+def from_cumulative_quantities(outcomes: Sequence,
+                               cumulative_quantities: Sequence[int],
+                               *,
+                               reverse: bool = False) -> 'icepool.Die':
+    """Constructs a `Die` from a sequence of cumulative quantities.
+
+    Args:
+        outcomes: The outcomes of the resulting die. Sorted order is recommended
+            but not necessary.
+        cumulative_quantities: The cumulative quantities (inclusive) of the
+            outcomes in the order they are given to this function.
+        reverse: Iff true, both of the arguments will be reversed. This allows
+            e.g. constructing using a survival distribution.
+    """
+
     prev = 0
     d = {}
-    for outcome, weight in zip(outcomes, cweights):
-        d[outcome] = weight - prev
-        prev = weight
-    return icepool.Die(d)
-
-
-def from_sweights(outcomes: Sequence, sweights: Sequence[int]) -> 'icepool.Die':
-    """Constructs a die from survival weights. """
-    prev = 0
-    d = {}
-    for outcome, weight in zip(reversed(outcomes), reversed(tuple(sweights))):
-        d[outcome] = weight - prev
-        prev = weight
+    for outcome, quantity in zip((reversed(outcomes) if reverse else outcomes),
+                                 (reversed(cumulative_quantities)
+                                  if reverse else cumulative_quantities)):
+        d[outcome] = quantity - prev
+        prev = quantity
     return icepool.Die(d)
 
 
 def from_rv(rv, outcomes: Sequence[int | float], denominator: int, **kwargs):
-    """Constructs a die from a rv object (as `scipy.stats`).
+    """Constructs a `Die` from a rv object (as `scipy.stats`).
     Args:
         rv: A rv object (as `scipy.stats`).
         outcomes: An iterable of `int`s or `float`s that will be the outcomes
-            of the resulting die.
+            of the resulting `Die`.
             If the distribution is discrete, outcomes must be `int`s.
-        denominator: The denominator of the resulting die will be set to this.
+        denominator: The denominator of the resulting `Die` will be set to this.
         **kwargs: These will be provided to `rv.cdf()`.
     """
     if hasattr(rv, 'pdf'):
         # Continuous distributions use midpoints.
         midpoints = [(a + b) / 2 for a, b in zip(outcomes[:-1], outcomes[1:])]
         cdf = rv.cdf(midpoints, **kwargs)
-        cweights = tuple(
+        quantities_le = tuple(
             int(round(x * denominator)) for x in cdf) + (denominator,)
     else:
         cdf = rv.cdf(outcomes, **kwargs)
-        cweights = tuple(int(round(x * denominator)) for x in cdf)
-    return from_cweights(outcomes, cweights)
+        quantities_le = tuple(int(round(x * denominator)) for x in cdf)
+    return from_cumulative_quantities(outcomes, quantities_le)
 
 
 def min_outcome(*dice):
-    """Returns the minimum possible outcome among the dice. """
+    """The minimum outcome among the dice. """
     dice = [icepool.Die([die]) for die in dice]
     return min(die.outcomes()[0] for die in dice)
 
 
 def max_outcome(*dice):
-    """Returns the maximum possible outcome among the dice. """
+    """The maximum outcome among the dice. """
     dice = [icepool.Die([die]) for die in dice]
     return max(die.outcomes()[-1] for die in dice)
 
 
 def align(*dice) -> tuple['icepool.Die', ...]:
-    """Pads dice with zero weights so that all have the same set of outcomes.
+    """Pads dice with zero quantities so that all have the same set of outcomes.
 
     Args:
-        *dice: One die per argument.
+        *dice: One `Die` per argument.
 
     Returns:
         A tuple of aligned dice.
@@ -127,10 +135,10 @@ def align(*dice) -> tuple['icepool.Die', ...]:
 
 
 def align_range(*dice) -> tuple['icepool.Die', ...]:
-    """Pads dice with zero weights so that all have the same set of consecutive `int` outcomes.
+    """Pads dice with zero quantities so that all have the same set of consecutive `int` outcomes.
 
     Args:
-        *dice: One die per argument.
+        *dice: One `Die` per argument.
 
     Returns:
         A tuple of aligned dice.
@@ -204,65 +212,65 @@ def accumulate(func: Callable[[Any, Any], Any], dice, *, initial=None):
 
 
 def apply(func: Callable, *dice) -> 'icepool.Die':
-    """Applies `func(outcome_of_die_0, outcome_of_die_1, ...)` for all possible outcomes of the dice.
+    """Applies `func(outcome_of_die_0, outcome_of_die_1, ...)` for all outcomes of the dice.
 
     Example: `apply(lambda a, b: a + b, d6, d6)` is the same as d6 + d6.
 
     `apply()` is flexible but not very efficient for more than two dice.
     Instead of using more than two arguments:
 
-    * If the problem is easy to solve by considering one additional die at a
+    * If the problem is easy to solve by considering one additional `Die` at a
         time, try using `reduce()` instead.
     * If the problem is easy to solve by considering how many dice rolled each
         outcome, one outcome at a time, try using
-        `icepool.Pool` and `icepool.EvalPool`.
+        `icepool.Pool` and `icepool.OutcomeCountEvaluator`.
     * If the order in which the dice are rolled is not important, you can use
         `apply_sorted()`. This is less efficient than either of the above two,
         but is still more efficient than `apply()`.
 
     Args:
-        func: A function that takes one argument per input die and returns an
+        func: A function that takes one argument per input `Die` and returns an
             argument to `Die()`.
         *dice: Any number of dice (or objects convertible to dice).
-            `func` will be called with all possible joint outcomes of `dice`,
-            with one argument per die.
+            `func` will be called with all joint outcomes of `dice`, with one
+            argument per `Die`.
 
     Returns:
-        A die constructed from the outputs of `func` and the product of the
-        weights of the dice.
+        A `Die` constructed from the outputs of `func` and the product of the
+        quantities of the dice.
     """
     if len(dice) == 0:
         return icepool.Die([func()])
     dice = tuple(icepool.Die([die]) for die in dice)
     final_outcomes = []
-    final_weights = []
+    final_quantities = []
     for t in itertools.product(*(die.items() for die in dice)):
-        outcomes, weights = zip(*t)
+        outcomes, quantities = zip(*t)
         final_outcome = func(*outcomes)
-        final_weight = math.prod(weights)
+        final_quantity = math.prod(quantities)
         if final_outcome is not icepool.Reroll:
             final_outcomes.append(final_outcome)
-            final_weights.append(final_weight)
+            final_quantities.append(final_quantity)
 
-    return icepool.Die(final_outcomes, final_weights)
+    return icepool.Die(final_outcomes, final_quantities)
 
 
 def apply_sorted(func: Callable, *dice) -> 'icepool.Die':
-    """Applies `func(lowest_outcome, next_lowest_outcome...)` for all possible sorted outcomes of the dice.
+    """Applies `func(lowest_outcome, next_lowest_outcome...)` for all sorted joint outcomes of the dice.
 
-    This is more efficient than `apply` but still not very efficient.
-    Use `EvalPool` instead if at all possible.
+    This is more efficient than `apply()` but still not very efficient.
+    Use `OutcomeCountEvaluator` instead if at all possible.
 
     Args:
-        func: A function that takes one argument per input die and returns an
+        func: A function that takes one argument per input `Die` and returns an
             argument to `Die()`.
         *dice: Any number of dice (or objects convertible to dice).
-            `func` will be called with all possible sorted outcomes of `dice`,
+            `func` will be called with all sorted joint outcomes of `dice`,
             with one argument per die. All outcomes must be totally orderable.
 
     Returns:
-        A die constructed from the outputs of `func` and the weight of rolling
+        A `Die` constructed from the outputs of `func` and the weight of rolling
         the corresponding sorted outcomes.
     """
     pool = icepool.Pool(dice)
-    return icepool.enumerate_gen(pool).sub(func, star=1)
+    return icepool.enumerate_sorted(pool).sub(func, star=1)
