@@ -5,25 +5,24 @@ import icepool.math
 import icepool.pool_cost
 import icepool.creation_args
 from icepool.counts import Counts
-from icepool.gen import GenGenerator, OutcomeCountGen
+from icepool.outcome_count_generator import NextOutcomeCountGenerator, OutcomeCountGenerator
 
 import itertools
 import math
 from collections import defaultdict
 from functools import cache, cached_property
 
-from typing import Any, Generator
-from collections.abc import Collection, Mapping, MutableMapping, Sequence
+from typing import Any, Collection, Generator, Mapping, MutableMapping, Sequence
 
 
 @cache
 def new_pool_cached(cls, dice: tuple[tuple['icepool.Die', int]],
-                    post_roll_counts: tuple[int, ...]) -> 'Pool':
-    """Creates a new pool. This function is cached.
+                    post_roll_counts: tuple[int, ...], /) -> 'Pool':
+    """Creates a new `Pool`. This function is cached.
 
     Args:
-        cls: The pool class.
-        sorted_num_dices: A sorted sequence of (die, num_dice) pairs.
+        cls: The `Pool` class.
+        dice: A sorted sequence of (die, rolls) pairs.
         post_roll_counts: A tuple of length equal to the number of dice.
     """
     self = super(Pool, cls).__new__(cls)
@@ -37,10 +36,10 @@ def clear_pool_cache():
     new_pool_cached.cache_clear()
 
 
-class Pool(OutcomeCountGen):
-    """Represents a set of unordered dice, only distinguished by the outcomes they roll.
+class Pool(OutcomeCountGenerator):
+    """Represents a set of sorted/unordered dice, only distinguished by the outcomes they roll.
 
-    This should be used in conjunction with `EvalPool` to generate a result.
+    This should be used in conjunction with `OutcomeCountEvaluator` to generate a result.
     """
 
     _post_roll_counts: tuple[int, ...]
@@ -56,12 +55,12 @@ class Pool(OutcomeCountGen):
         same-side truncations of d12.
 
         Args:
-            dice: The dice to put in the pool. This can be one of the following:
+            dice: The dice to put in the `Pool`. This can be one of the following:
 
                 * A sequence of dice.
-                * A mapping of dice and how many of that die to put in the pool.
+                * A mapping of dice and how many of that `Die` to put in the `Pool`.
 
-                All outcomes within a pool must be totally orderable.
+                All outcomes within a `Pool` must be totally orderable.
             times: Multiplies the number of times each element of `dice` will
                 be put into the pool.
                 `times` can either be a sequence of the same length as
@@ -69,8 +68,8 @@ class Pool(OutcomeCountGen):
                 `outcomes`.
 
         Raises:
-            ValueError for a `Deck` or `Die` argument. A pool of a single `Die`
-                should constructed as `Pool([die])`.
+            ValueError: If a bare `Deck` or `Die` argument is provided.
+                A `Pool` of a single `Die` should constructed as `Pool([die])`.
         """
         if isinstance(dice, Pool):
             if times == 1:
@@ -104,7 +103,7 @@ class Pool(OutcomeCountGen):
         """Creates a new pool.
 
         Args:
-            dice_counts: A map from dice to num_dice.
+            dice_counts: A map from dice to rolls.
             post_roll_counts: A tuple with length equal to the number of dice.
         """
         dice = tuple(
@@ -129,19 +128,18 @@ class Pool(OutcomeCountGen):
             post_roll_counts)
 
     @cached_property
-    def _num_dice(self) -> int:
+    def _size(self) -> int:
         return sum(count for _, count in self._dice)
 
-    def num_dice(self) -> int:
-        """The number of dice in this pool."""
-        return self._num_dice
+    def size(self) -> int:
+        """The number of dice in this pool, counting multiples of the same `Die`."""
+        return self._size
 
     def _is_resolvable(self) -> bool:
         return all(not die.is_empty() for die, _ in self._dice)
 
     @cached_property
     def _denominator(self) -> int:
-        """The product of the total dice weights in this pool."""
         return math.prod(die.denominator()**count for die, count in self._dice)
 
     def denominator(self) -> int:
@@ -156,6 +154,7 @@ class Pool(OutcomeCountGen):
         return tuple(die for die, _ in self._dice)
 
     def unique_dice(self) -> Collection['icepool.Die']:
+        """The collection of unique dice in this pool."""
         return self._unique_dice
 
     @cached_property
@@ -181,21 +180,21 @@ class Pool(OutcomeCountGen):
     def post_roll_counts(self) -> tuple[int, ...]:
         """The tuple indicating which dice in the pool will be counted.
 
-        The tuple has one element per die in the pool, from lowest roll to
+        The tuple has one element per `Die` in the pool, from lowest roll to
         highest roll.
         """
         return self._post_roll_counts
 
     def set_post_roll_counts(self,
                              post_roll_counts: int | slice | tuple[int, ...]):
-        """Returns a pool with the selected dice counted after rolling and sorting.
+        """A `Pool` with the selected dice counted after rolling and sorting.
 
-        You can use `pool[post_roll_counts]` for the same effect as this method.
+        Use `pool[post_roll_counts]` for the same effect as this method.
 
         The dice are sorted in ascending order for this purpose,
         regardless of which order the outcomes are evaluated in.
 
-        This is always an absolute selection on all `num_dice`,
+        This is always an absolute selection on all `size` dice,
         not a relative selection on already-selected dice,
         which would be ambiguous in the presence of multiple or negative counts.
 
@@ -207,7 +206,7 @@ class Pool(OutcomeCountGen):
         * `pool[..., 1, 1]`
 
         These will also select the two highest dice out of 5, and will also
-        resize the pool to 5 dice first:
+        resize the `Pool` to 5 dice first:
 
         * `pool[3::5]`
         * `pool[3:5:5]`
@@ -220,48 +219,47 @@ class Pool(OutcomeCountGen):
         * `pool[-1, ..., 1]`
 
         Args:
-            An `int`. This will count only the die at the specified index (once).
+            An `int`. This will count only the `Die` at the specified index (once).
                 In this case, the result will be a `Die`, not a pool.
             A `slice`. The selected dice are counted once each.
                 If provided, the third argument resizes the pool,
                 rather than being a step; however, this is only valid if the
-                pool consists of a single type of die.
-            A sequence of one `int` for each die.
-                Each die is counted that many times, which could be multiple or
+                `Pool` consists of a single type of `Die`.
+            A sequence of one `int` for each `Die`.
+                Each `Die` is counted that many times, which could be multiple or
                 negative times. This may resize the pool, but only if the
-                pool consists of a single type of die.
+                `Pool` consists of a single type of `Die`.
 
                 Up to one `Ellipsis` (`...`) may be used.
-                If an `Ellipsis` is used, the size of the pool won't change.
+                If an `Ellipsis` is used, the size of the `Pool` won't change.
                 Instead, the `Ellipsis` will be replaced with a number of zero
-                counts, sufficient to maintain the current size of this pool.
+                counts, sufficient to maintain the current size of this `Pool`.
                 This number may be "negative" if more `int`s are provided than
-                the size of the pool. Specifically:
+                the size of the `Pool`. Specifically:
 
-                * If `post_roll_counts` is shorter than `num_dice`, the `Ellipsis`
+                * If `post_roll_counts` is shorter than `size`, the `Ellipsis`
                     acts as enough zero counts to make up the difference.
                     E.g. `pool[1, ..., 1]` on five dice would act as `pool[1, 0, 0, 0, 1]`.
-                * If `post_roll_counts` has length equal to `num_dice`, the `Ellipsis` has no effect.
+                * If `post_roll_counts` has length equal to `size`, the `Ellipsis` has no effect.
                     E.g. `pool[1, ..., 1]` on two dice would act as `pool[1, 1]`.
-                * If `post_roll_counts` is longer than `num_dice` and the `Ellipsis` is on one side,
+                * If `post_roll_counts` is longer than `size` and the `Ellipsis` is on one side,
                     elements will be dropped from `post_roll_counts` on the side with the `Ellipsis`.
                     E.g. `pool[..., 1, 2, 3]` on two dice would act as `pool[2, 3]`.
-                * If `post_roll_counts` is longer than `num_dice` and the `Ellipsis`
+                * If `post_roll_counts` is longer than `size` and the `Ellipsis`
                     is in the middle, the counts will be as the sum of two
                     one-sided `Ellipsis`.
                     E.g. `pool[-1, ..., 1]` acts like `[-1, ...]` plus `[..., 1]`.
-                    On a pool consisting of a single single die this would have
+                    On a `Pool` consisting of a single single `Die` this would have
                     the -1 and 1 cancel each other out.
 
         Raises:
-            ValueError:  If `post_roll_counts` would change the size of a pool with
-                more than one type of die, or if more than one `Ellipsis`
+            ValueError:  If `post_roll_counts` would change the size of a `Pool`
+                with more than one type of `Die`, or if more than one `Ellipsis`
                 is used.
         """
         convert_to_die = isinstance(post_roll_counts, int)
-        post_roll_counts = post_roll_counts_tuple(self.num_dice(),
-                                                  post_roll_counts)
-        if len(post_roll_counts) != self.num_dice():
+        post_roll_counts = post_roll_counts_tuple(self.size(), post_roll_counts)
+        if len(post_roll_counts) != self.size():
             if len(self._dice) != 1:
                 raise ValueError(
                     'Cannot change the size of a pool unless it has exactly one type of die.'
@@ -271,8 +269,8 @@ class Pool(OutcomeCountGen):
         else:
             result = Pool._new_pool_from_tuple(self._dice, post_roll_counts)
         if convert_to_die:
-            return result.eval(lambda state, outcome, count: outcome
-                               if count else state)
+            return result.evaluate(lambda state, outcome, count: outcome
+                                   if count else state)
         else:
             return result
 
@@ -283,7 +281,7 @@ class Pool(OutcomeCountGen):
         return min(die.min_outcome() for die in self.unique_dice())
 
     def min_outcome(self):
-        """Returns the min outcome among all dice in this pool."""
+        """The min outcome among all dice in this pool."""
         return self._min_outcome
 
     @cached_property
@@ -291,10 +289,10 @@ class Pool(OutcomeCountGen):
         return max(die.max_outcome() for die in self.unique_dice())
 
     def max_outcome(self):
-        """Returns the max outcome among all dice in this pool."""
+        """The max outcome among all dice in this pool."""
         return self._max_outcome
 
-    def _gen_min(self, min_outcome) -> GenGenerator:
+    def _generate_min(self, min_outcome) -> NextOutcomeCountGenerator:
         """Pops the given outcome from this pool, if it is the min outcome.
 
         Yields:
@@ -312,21 +310,20 @@ class Pool(OutcomeCountGen):
         ]
         skip_weight = None
         for pop in itertools.product(*generators):
-            net_num_rolled = 0
+            total_hits = 0
             result_weight = 1
             next_dice_counts: MutableMapping[Any, int] = defaultdict(int)
-            for popped_die, num_remain, num_rolled, weight in pop:
+            for popped_die, misses, hits, weight in pop:
                 if not popped_die.is_empty():
-                    next_dice_counts[popped_die] += num_remain
-                net_num_rolled += num_rolled
+                    next_dice_counts[popped_die] += misses
+                total_hits += hits
                 result_weight *= weight
-            if net_num_rolled == 0:
+            if total_hits == 0:
                 result_count = 0
                 popped_post_roll_counts = self.post_roll_counts()
             else:
-                result_count = sum(self.post_roll_counts()[:net_num_rolled])
-                popped_post_roll_counts = self.post_roll_counts(
-                )[net_num_rolled:]
+                result_count = sum(self.post_roll_counts()[:total_hits])
+                popped_post_roll_counts = self.post_roll_counts()[total_hits:]
             popped_pool = Pool._new_pool_from_mapping(next_dice_counts,
                                                       popped_post_roll_counts)
             if not any(popped_post_roll_counts):
@@ -340,7 +337,7 @@ class Pool(OutcomeCountGen):
         if skip_weight is not None:
             yield Pool([]), (sum(self.post_roll_counts()),), skip_weight
 
-    def _gen_max(self, max_outcome) -> GenGenerator:
+    def _generate_max(self, max_outcome) -> NextOutcomeCountGenerator:
         """Pops the given outcome from this pool, if it is the max outcome.
 
         Yields:
@@ -358,21 +355,20 @@ class Pool(OutcomeCountGen):
         ]
         skip_weight = None
         for pop in itertools.product(*generators):
-            net_num_rolled = 0
+            total_hits = 0
             result_weight = 1
             next_dice_counts: MutableMapping[Any, int] = defaultdict(int)
-            for popped_die, num_remain, num_rolled, weight in pop:
+            for popped_die, misses, hits, weight in pop:
                 if not popped_die.is_empty():
-                    next_dice_counts[popped_die] += num_remain
-                net_num_rolled += num_rolled
+                    next_dice_counts[popped_die] += misses
+                total_hits += hits
                 result_weight *= weight
-            if net_num_rolled == 0:
+            if total_hits == 0:
                 result_count = 0
                 popped_post_roll_counts = self.post_roll_counts()
             else:
-                result_count = sum(self.post_roll_counts()[-net_num_rolled:])
-                popped_post_roll_counts = self.post_roll_counts(
-                )[:-net_num_rolled]
+                result_count = sum(self.post_roll_counts()[-total_hits:])
+                popped_post_roll_counts = self.post_roll_counts()[:-total_hits]
             popped_pool = Pool._new_pool_from_mapping(next_dice_counts,
                                                       popped_post_roll_counts)
             if not any(popped_post_roll_counts):
@@ -386,47 +382,47 @@ class Pool(OutcomeCountGen):
         if skip_weight is not None:
             yield Pool([]), (sum(self.post_roll_counts()),), skip_weight
 
-    def lowest(self, num_keep: int = 1, num_drop: int = 0) -> 'icepool.Die':
+    def lowest(self, keep: int = 1, drop: int = 0) -> 'icepool.Die':
         """The lowest outcome or sum of the lowest outcomes in the pool.
 
         The args override any `post_roll_counts` of this pool.
 
         Args:
-            num_keep: The number of lowest dice will be summed.
-            num_drop: This number of lowest dice will be dropped before keeping
+            keep: The number of lowest dice will be summed.
+            drop: This number of lowest dice will be dropped before keeping
                 dice to be summed.
         """
-        if num_keep < 0:
-            raise ValueError(f'num_drop={num_keep} cannot be negative.')
-        if num_drop < 0:
-            raise ValueError(f'num_drop={num_drop} cannot be negative.')
+        if keep < 0:
+            raise ValueError(f'keep={keep} cannot be negative.')
+        if drop < 0:
+            raise ValueError(f'drop={drop} cannot be negative.')
 
-        start = min(num_drop, self.num_dice())
-        stop = min(num_keep + num_drop, self.num_dice())
+        start = min(drop, self.size())
+        stop = min(keep + drop, self.size())
         return self[start:stop].sum()  # type: ignore
 
-    def highest(self, num_keep: int = 1, num_drop: int = 0) -> 'icepool.Die':
+    def highest(self, keep: int = 1, drop: int = 0) -> 'icepool.Die':
         """The highest outcome or sum of the highest outcomes in the pool.
 
         The args override any `post_roll_counts` of this pool.
 
         Args:
-            num_keep: The number of highest dice will be summed.
-            num_drop: This number of highest dice will be dropped before keeping
+            keep: The number of highest dice will be summed.
+            drop: This number of highest dice will be dropped before keeping
                 dice to be summed.
         """
-        if num_keep < 0:
-            raise ValueError(f'num_drop={num_keep} cannot be negative.')
-        if num_drop < 0:
-            raise ValueError(f'num_drop={num_drop} cannot be negative.')
+        if keep < 0:
+            raise ValueError(f'keep={keep} cannot be negative.')
+        if drop < 0:
+            raise ValueError(f'drop={drop} cannot be negative.')
 
-        start = self.num_dice() - min(num_keep + num_drop, self.num_dice())
-        stop = self.num_dice() - min(num_drop, self.num_dice())
+        start = self.size() - min(keep + drop, self.size())
+        stop = self.size() - min(drop, self.size())
         return self[start:stop].sum()  # type: ignore
 
     def __str__(self) -> str:
         return (
-            f'Pool of {self.num_dice()} dice with post_roll_counts={self.post_roll_counts()}\n'
+            f'Pool of {self.size()} dice with post_roll_counts={self.post_roll_counts()}\n'
             + ''.join(f'  {repr(die)}\n' for die in self._dice_tuple))
 
     @cached_property
@@ -447,32 +443,32 @@ class Pool(OutcomeCountGen):
 
 
 def post_roll_counts_tuple(
-        num_dice: int,
+        pool_size: int,
         post_roll_counts: int | slice | tuple[int, ...]) -> tuple[int, ...]:
     """Expresses `post_roll_counts` as a tuple.
 
     See `Pool.set_post_roll_counts()` for details.
 
     Args:
-        `num_dice`: An `int` specifying the number of dice.
+        `pool_size`: An `int` specifying the size of the pool.
         `post_roll_counts`: Raw specification for how the dice are to be counted.
     Raises:
-        `ValueError` if:
+        ValueError: If:
             * More than one `Ellipsis` is used.
-            * An `Ellipsis` is used in the center with too few `num_dice`.
+            * An `Ellipsis` is used in the center with too few `pool_size`.
     """
     if isinstance(post_roll_counts, int):
-        result = [0] * num_dice
+        result = [0] * pool_size
         result[post_roll_counts] = 1
         return tuple(result)
     elif isinstance(post_roll_counts, slice):
         if post_roll_counts.step is not None:
             # "Step" is not useful here, so we repurpose it to set the number
             # of dice.
-            num_dice = post_roll_counts.step
+            pool_size = post_roll_counts.step
             post_roll_counts = slice(post_roll_counts.start,
                                      post_roll_counts.stop)
-        result = [0] * num_dice
+        result = [0] * pool_size
         result[post_roll_counts] = [1] * len(result[post_roll_counts])
         return tuple(result)
     else:
@@ -489,7 +485,7 @@ def post_roll_counts_tuple(
         if split is None:
             return tuple(post_roll_counts)
 
-        extra_dice = num_dice - len(post_roll_counts) + 1
+        extra_dice = pool_size - len(post_roll_counts) + 1
 
         if split == 0:
             # Ellipsis on left.
@@ -508,11 +504,11 @@ def post_roll_counts_tuple(
         else:
             # Ellipsis in center.
             if extra_dice < 0:
-                result = [0] * num_dice
-                for i in range(min(split, num_dice)):
+                result = [0] * pool_size
+                for i in range(min(split, pool_size)):
                     result[i] += post_roll_counts[i]
                 reverse_split = split - len(post_roll_counts)
-                for i in range(-1, max(reverse_split - 1, -num_dice - 1), -1):
+                for i in range(-1, max(reverse_split - 1, -pool_size - 1), -1):
                     result[i] += post_roll_counts[i]
                 return tuple(result)
             else:
@@ -522,87 +518,87 @@ def post_roll_counts_tuple(
 
 
 def standard_pool(die_sizes: Collection[int]) -> 'Pool':
-    """Returns a pool of standard dice.
+    """A `Pool` of standard dice (e.g. d6, d8...).
 
     Args:
-        die_sizes: For each of these die_size X, the pool will contain one dX.
+        die_sizes: For each of these die_size X, the `Pool` will contain one dX.
     """
     return Pool(list(icepool.d(x) for x in die_sizes))
 
 
 def iter_die_pop_min(
-        die: 'icepool.Die', num_dice: int, min_outcome
+        die: 'icepool.Die', rolls: int, min_outcome
 ) -> Generator[tuple['icepool.Die', int, int, int], None, None]:
     """Helper function to iterate over the possibilities of several identical dice rolling a min outcome.
 
     Args:
-        die: The die to pop.
-        die_count: The number of this kind of die.
-        min_outcome: The outcome to pop. This is <= the die's min outcome.
+        die: The `Die` to pop.
+        rolls: The number of this kind of `Die`.
+        min_outcome: The outcome to pop. This is <= the `Die`'s min outcome.
 
     Yields:
-        The popped die.
-        The number of remaining dice of this kind.
-        The number of dice that rolled max_outcome.
-        The weight of this number of dice rolling max_outcome.
+        popped_die
+        misses: The number of dice that didn't roll this outcome.
+        hits: The number of dice that rolled this outcome.
+        weight: The weight of this number of dice rolling max_outcome.
     """
     if die.min_outcome() != min_outcome:
-        num_remain = num_dice
-        num_rolled = 0
+        misses = rolls
+        hits = 0
         weight = 1
-        yield die, num_remain, num_rolled, weight
+        yield die, misses, hits, weight
         return
 
     popped_die, single_weight = die._pop_min()
 
     if popped_die.is_empty():
         # This is the last outcome. All dice must roll this outcome.
-        num_remain = 0
-        num_rolled = num_dice
-        weight = single_weight**num_dice
-        yield popped_die, num_remain, num_rolled, weight
+        misses = 0
+        hits = rolls
+        weight = single_weight**rolls
+        yield popped_die, misses, hits, weight
         return
 
-    comb_row = icepool.math.comb_row(num_dice, single_weight)
-    for num_rolled, weight in enumerate(comb_row):
-        num_remain = num_dice - num_rolled
-        yield popped_die, num_remain, num_rolled, weight
+    comb_row = icepool.math.comb_row(rolls, single_weight)
+    for hits, weight in enumerate(comb_row):
+        misses = rolls - hits
+        yield popped_die, misses, hits, weight
 
 
 def iter_die_pop_max(
-        die: 'icepool.Die', num_dice: int, max_outcome
+        die: 'icepool.Die', rolls: int, max_outcome
 ) -> Generator[tuple['icepool.Die', int, int, int], None, None]:
     """Helper function to iterate over the possibilities of several identical dice rolling a max outcome.
 
     Args:
-        die: The die to pop.
-        die_count: The number of this kind of die.
-        max_outcome: The outcome to pop. This is >= the die's max outcome.
+        die: The `Die` to pop.
+        rolls: The number of this kind of `Die`.
+        max_outcome: The outcome to pop. This is >= the `Die`'s max outcome.
 
     Yields:
-        The popped die.
-        The number of remaining dice of this kind.
-        The number of dice that rolled max_outcome.
-        The weight of this number of dice rolling max_outcome.
+        popped_die
+        misses: The number of dice that didn't roll this outcome.
+        hits: The number of dice that rolled this outcome.
+        weight: The weight of this number of dice rolling max_outcome.
     """
     if die.max_outcome() != max_outcome:
-        num_remain = num_dice
-        num_rolled = 0
+        misses = rolls
+        hits = 0
         weight = 1
-        yield die, num_remain, num_rolled, weight
+        yield die, misses, hits, weight
         return
 
     popped_die, single_weight = die._pop_max()
 
     if popped_die.is_empty():
         # This is the last outcome. All dice must roll this outcome.
-        num_remain = 0
-        num_rolled = num_dice
-        weight = single_weight**num_dice
-        yield popped_die, num_remain, num_rolled, weight
+        misses = 0
+        hits = rolls
+        weight = single_weight**rolls
+        yield popped_die, misses, hits, weight
         return
 
-    comb_row = icepool.math.comb_row(num_dice, single_weight)
-    for num_rolled, weight in enumerate(comb_row):
-        num_remain = num_dice - num_rolled
-        yield popped_die, num_remain, num_rolled, weight
+    comb_row = icepool.math.comb_row(rolls, single_weight)
+    for hits, weight in enumerate(comb_row):
+        misses = rolls - hits
+        yield popped_die, misses, hits, weight
